@@ -1,8 +1,7 @@
-import sqlite3
+import sqlite3; sqlite3.enable_callback_tracebacks(True)
 import unittest
 
 from peewee import *
-from peewee import Clause
 from playhouse import sqlite_ext as sqe
 
 # use a disk-backed db since memory dbs only exist for a single connection and
@@ -78,12 +77,22 @@ class ManagedDoc(sqe.FTSModel):
     class Meta:
         database = ext_db
 
+class MultiColumn(sqe.FTSModel):
+    c1 = CharField(default='')
+    c2 = CharField(default='')
+    c3 = CharField(default='')
+    c4 = IntegerField()
+
+    class Meta:
+        database = ext_db
+
 class Values(BaseExtModel):
     klass = IntegerField()
     value = FloatField()
     weight = FloatField()
 
 
+
 class SqliteExtTestCase(unittest.TestCase):
     messages = [
         'A faith is a necessity to a man. Woe to him who believes in nothing.',
@@ -93,13 +102,22 @@ class SqliteExtTestCase(unittest.TestCase):
         'Faith consists in believing when it is beyond the power of reason to believe.',
         'Faith has to do with things that are not seen and hope with things that are not at hand.',
     ]
+    values = [
+        ('aaaaa bbbbb ccccc ddddd', 'aaaaa ccccc', 'zzzzz zzzzz', 1),
+        ('bbbbb ccccc ddddd eeeee', 'bbbbb', 'zzzzz', 2),
+        ('ccccc ccccc ddddd fffff', 'ccccc', 'yyyyy', 3),
+        ('ddddd', 'ccccc', 'xxxxx', 4),
+    ]
+
     def setUp(self):
         FTSDoc.drop_table(True)
         ManagedDoc.drop_table(True)
         FTSPost.drop_table(True)
         Post.drop_table(True)
+        MultiColumn.drop_table(True)
         Values.drop_table(True)
         Values.create_table()
+        MultiColumn.create_table(tokenize='porter')
         Post.create_table()
         FTSPost.create_table(tokenize='porter', content=Post)
         ManagedDoc.create_table(tokenize='porter', content=Post.message)
@@ -123,61 +141,172 @@ class SqliteExtTestCase(unittest.TestCase):
             self.messages[i] for i in indices])
 
     def test_fts_manual(self):
-        matches = lambda s: sqe.match(FTSDoc.message, s)
         messages = [FTSDoc.create(message=msg) for msg in self.messages]
 
-        q = FTSDoc.select().where(matches('believe')).order_by(FTSDoc.id)
+        q = FTSDoc.select().where(FTSDoc.match('believe')).order_by(FTSDoc.id)
         self.assertMessages(q, [0, 3])
 
-        q = FTSDoc.match('believe')
+        q = FTSDoc.search('believe')
         self.assertMessages(q, [3, 0])
 
-        q = FTSDoc.match('things')
+        q = FTSDoc.search('things')
         self.assertEqual([(x.message, x.score) for x in q], [
             (self.messages[4], 2.0 / 3),
             (self.messages[2], 1.0 / 3),
         ])
 
+    def _create_multi_column(self):
+        for c1, c2, c3, c4 in self.values:
+            MultiColumn.create(c1=c1, c2=c2, c3=c3, c4=c4)
+
+    def test_fts_multi_column(self):
+        def assertResults(term, expected):
+            results = [
+                (x.c4, round(x.score, 2))
+                for x in MultiColumn.search(term)]
+            self.assertEqual(results, expected)
+
+        self._create_multi_column()
+
+        # `bbbbb` appears two times in `c1`, one time in `c2`.
+        assertResults('bbbbb', [
+            (2, 1.5),  # 1/2 + 1/1
+            (1, 0.5),  # 1/2
+        ])
+
+        # `ccccc` appears four times in `c1`, three times in `c2`.
+        assertResults('ccccc', [
+            (3, .83),  # 2/4 + 1/3
+            (1, .58), # 1/4 + 1/3
+            (4, .33), # 1/3
+            (2, .25), # 1/4
+        ])
+
+        # `zzzzz` appears three times in c3.
+        assertResults('zzzzz', [
+            (1, .67),
+            (2, .33),
+        ])
+
+        self.assertEqual(
+            [x.score for x in MultiColumn.search('ddddd')],
+            [.25, .25, .25, .25])
+
+    def test_bm25(self):
+        def assertResults(term, col_idx, expected):
+            query = MultiColumn.search_bm25(term, MultiColumn.c1)
+            self.assertEqual(
+                [(mc.c4, round(mc.score, 2)) for mc in query],
+                expected)
+
+        self._create_multi_column()
+        MultiColumn.create(c1='aaaaa fffff', c4=5)
+
+        assertResults('aaaaa', 1, [
+            (5, 0.39),
+            (1, 0.3),
+        ])
+        assertResults('fffff', 1, [
+            (5, 0.39),
+            (3, 0.3),
+        ])
+        assertResults('eeeee', 1, [
+            (2, 0.97),
+        ])
+
+        # No column specified, use the first text field.
+        query = MultiColumn.search_bm25('fffff')
+        self.assertEqual([(mc.c4, round(mc.score, 2)) for mc in query], [
+            (5, 0.39),
+            (3, 0.3),
+        ])
+
+        # Use helpers.
+        query = (MultiColumn
+                 .select(
+                     MultiColumn.c4,
+                     MultiColumn.bm25(MultiColumn.c1).alias('score'))
+                 .where(MultiColumn.match('aaaaa'))
+                 .order_by(SQL('score').desc()))
+        self.assertEqual([(mc.c4, round(mc.score, 2)) for mc in query], [
+            (5, 0.39),
+            (1, 0.3),
+        ])
+
+    def test_bm25_alt_corpus(self):
+        for message in self.messages:
+            FTSDoc.create(message=message)
+
+        def assertResults(term, expected):
+            query = FTSDoc.search_bm25(term)
+            cleaned = [
+                (round(doc.score, 2), ' '.join(doc.message.split()[:2]))
+                for doc in query]
+            self.assertEqual(cleaned, expected)
+
+        assertResults('things', [
+            (0.45, 'Faith has'),
+            (0.36, 'Be faithful'),
+        ])
+
+        # Indeterminate order since all are 0.0. All phrases contain the word
+        # faith, so there is no meaningful score.
+        results = [x.score for x in FTSDoc.search_bm25('faith')]
+        self.assertEqual(results, [0., 0., 0., 0., 0.])
+
     def _test_fts_auto(self, ModelClass):
-        matches = lambda s: sqe.match(ModelClass.message, s)
         posts = []
         for message in self.messages:
             posts.append(Post.create(message=message))
 
         # Nothing matches, index is not built.
-        pq = ModelClass.select().where(matches('faith'))
+        pq = ModelClass.select().where(ModelClass.match('faith'))
         self.assertEqual(list(pq), [])
 
         ModelClass.rebuild()
         ModelClass.optimize()
 
         # it will stem faithful -> faith b/c we use the porter tokenizer
-        pq = ModelClass.select().where(matches('faith')).order_by(ModelClass.id)
+        pq = (ModelClass
+              .select()
+              .where(ModelClass.match('faith'))
+              .order_by(ModelClass.id))
         self.assertMessages(pq, range(len(self.messages)))
 
-        pq = ModelClass.select().where(matches('believe')).order_by(ModelClass.id)
+        pq = (ModelClass
+              .select()
+              .where(ModelClass.match('believe'))
+              .order_by(ModelClass.id))
         self.assertMessages(pq, [0, 3])
 
-        pq = ModelClass.select().where(matches('thin*')).order_by(ModelClass.id)
+        pq = (ModelClass
+              .select()
+              .where(ModelClass.match('thin*'))
+              .order_by(ModelClass.id))
         self.assertMessages(pq, [2, 4])
 
-        pq = ModelClass.select().where(matches('"it is"')).order_by(ModelClass.id)
+        pq = (ModelClass
+              .select()
+              .where(ModelClass.match('"it is"'))
+              .order_by(ModelClass.id))
         self.assertMessages(pq, [2, 3])
 
-        pq = (ModelClass
-              .select(ModelClass, sqe.Rank(ModelClass).alias('score'))
-              .where(matches('things'))
-              .order_by(SQL('score').desc()))
+        pq = ModelClass.search('things')
         self.assertEqual([(x.message, x.score) for x in pq], [
             (self.messages[4], 2.0 / 3),
             (self.messages[2], 1.0 / 3),
         ])
 
-        pq = ModelClass.select(sqe.Rank(ModelClass)).where(matches('faithful')).tuples()
+        pq = (ModelClass
+              .select(sqe.Rank(ModelClass))
+              .where(ModelClass.match('faithful'))
+              .tuples())
         self.assertEqual([x[0] for x in pq], [.2] * 5)
 
-        pq = ModelClass.select(ModelClass.rank()).where(matches('faithful')).tuples()
-        self.assertEqual([x[0] for x in pq], [.2] * 5)
+        pq = (ModelClass
+              .search('faithful')
+              .dicts())
+        self.assertEqual([x['score'] for x in pq], [.2] * 5)
 
     def test_fts_auto_model(self):
         self._test_fts_auto(FTSPost)
