@@ -33,14 +33,17 @@ def get_option_parser():
     cases.add_option('--apsw', dest='apsw', default=False, action='store_true', help='apsw tests (requires apsw)')
     cases.add_option('--berkeleydb', dest='berkeleydb', default=False, action='store_true', help='berkeleydb tests (requires pysqlite compiled against berkeleydb)')
     cases.add_option('--csv', dest='csv', default=False, action='store_true', help='csv tests')
+    cases.add_option('--dataset', dest='dataset', default=False, action='store_true', help='dataset tests')
+    cases.add_option('--db-url', dest='db_url', default=False, action='store_true', help='db url tests')
     cases.add_option('--djpeewee', dest='djpeewee', default=False, action='store_true', help='djpeewee tests')
     cases.add_option('--gfk', dest='gfk', default=False, action='store_true', help='gfk tests')
     cases.add_option('--kv', dest='kv', default=False, action='store_true', help='key/value store tests')
     cases.add_option('--migrations', dest='migrations', default=False, action='store_true', help='migration helper tests (requires psycopg2)')
     cases.add_option('--pool', dest='pool', default=False, action='store_true', help='connection pool tests')
     cases.add_option('--postgres-ext', dest='postgres_ext', default=False, action='store_true', help='postgres_ext tests (requires psycopg2)')
-    cases.add_option('--pwiz', dest='pwiz', default=False, action='store_true', help='pwiz, schema introspector and model generator')
+    cases.add_option('--pwiz', dest='pwiz', default=False, action='store_true', help='pwiz, model code generator')
     cases.add_option('--read-slave', dest='read_slave', default=False, action='store_true', help='read_slave tests')
+    cases.add_option('--reflection', dest='reflection', default=False, action='store_true', help='reflection schema introspector')
     cases.add_option('--signals', dest='signals', default=False, action='store_true', help='signals tests')
     cases.add_option('--shortcuts', dest='shortcuts', default=False, action='store_true', help='shortcuts tests')
     cases.add_option('--sqlcipher-ext', dest='sqlcipher', default=False, action='store_true', help='sqlcipher_ext tests (requires pysqlcipher)')
@@ -68,8 +71,14 @@ def collect_modules(options):
         except ImportError:
             print_('Unable to import berkeleydb tests, skipping')
     if xtra(options.csv):
-        from playhouse import tests_csv_loader
-        modules.append(tests_csv_loader)
+        from playhouse import tests_csv_utils
+        modules.append(tests_csv_utils)
+    if xtra(options.dataset):
+        from playhouse import tests_dataset
+        modules.append(tests_dataset)
+    if xtra(options.db_url):
+        from playhouse import tests_db_url
+        modules.append(tests_db_url)
     if xtra(options.djpeewee):
         from playhouse import tests_djpeewee
         modules.append(tests_djpeewee)
@@ -103,6 +112,9 @@ def collect_modules(options):
     if xtra(options.read_slave):
         from playhouse import tests_read_slave
         modules.append(tests_read_slave)
+    if xtra(options.reflection):
+        from playhouse import tests_reflection
+        modules.append(tests_reflection)
     if xtra(options.signals):
         from playhouse import tests_signals
         modules.append(tests_signals)
